#include "catmull.h"

/* given 4 points (1, 2, 3, 4) on a curve or motion path, 
   catmull-rom will interpolate a point between 
   points 2 and 3 to smooth out motion.  To get a point 
   between 1 and 2, pass in original points (1, 1, 2, 3)
   and pass in points (2, 3, 4, 4) to interpolate 
   between points 3 and 4.  The x, y, and z values of the
   new point are stored in the x, y, and z fields of dest.
*/

void catmull_rom3d(ps2_vu0_fvector dest,
                   const ps2_vu0_fvector cr,
                   ps2_vu0_fvector x,
                   ps2_vu0_fvector y,
                   ps2_vu0_fvector z)
{
  const int one = 0;

  asm __volatile__("
  lqc2 $vf10,0x0(%1)  # load x vector         
  lqc2 $vf14,0x0(%4)  # load cr_consts   
  lw   $10, 0x0(%6)   # only do one mul/horizontal add
  ctc2 $10,$vi01      # load the counter
  vcallms %5          # process x vector
  nop                 # wait until vf10 isnt being used
  nop
  nop
  nop
  lqc2 $vf10, 0x0(%2) # load y vector
  vnop
  ctc2 $10,$vi01      # load the counter
  vmr32 $vf1,$vf1     # rotate the result
  vcallms %5         
  nop
  nop
  nop
  nop                
  lqc2 $vf10, 0x0(%3) #load z vector 
  vnop
  ctc2 $10,$vi01    
  vmr32 $vf1,$vf1           
  vcallms %5         
  vnop              
  vmr32 $vf1,$vf1     # rotate until x is in the x vector field
  vmr32 $vf1,$vf1
  sqc2 $vf1, 0x0(%0)  # store the result   
  "::"r"(dest), "r"(x), "r"(y), "r"(z), "r"(cr), "i"(vu0_mha), "r"(&one));
}
  
