#include "ps2init.h"

/*
   note- much thanks to colin for reminding me that micromem
   maps to main memory and I don't have to do all this through
   DMA channel 0 like I originally had planned
*/

int ps2init(char *microfile, ps2hardware *ps2)
{
  int f;

  /* we could open the GS here but this is an AI engine... 
     I'll worry about screenmode and graphics later.
  */
  
  /* open the vector units */
  if ((ps2->vu0 = open(PS2_DEV_VPU0,O_RDWR)) < 0)
     DBG("failed to open vu0",errno)

  if ((ps2->vu1 = open(PS2_DEV_VPU1,O_RDWR)) < 0)
     DBG("failed to open vu1",errno)

  /* reset vu0 */
  VU0_Reset();

  /* map vu's micromemory amd instructions to main memory */
  if ((ps2->vu0mem = mmap(NULL,VU0_SIZE,PROT_READ|PROT_WRITE,
                          MAP_SHARED,ps2->vu0,0)) < (void*)0)
     DBG("failed to mmap vu0",errno)

  if ((ps2->vu1mem = mmap(NULL,VU1_SIZE,PROT_READ|PROT_WRITE,
                          MAP_SHARED,ps2->vu1,0)) < (void*)0)
     DBG("failed to mmap vu1",errno)

  /* load microcode */
  if ((f = open(microfile,O_RDONLY)) < 0)
     DBG("failed to open microcode",errno)
  else
  {   
     read(f,ps2->vu0mem,VU0_SIZE/2);
     close(f);
  }

  /* open the (hopefully) dual shock 2 controller */
  if ((ps2->pad0 = open(PS2_DEV_PAD0,O_RDWR)) < 0)
     DBG("failed to open controller",errno)

  /* open scratchpad ram and try to map it */
  if ((ps2->spr = open(PS2_DEV_SPR,O_RDWR)) < 0)
     DBG("failed to open spram",errno)
  else
    if (mmap(SPRAM_ADDR,16384,PROT_READ|PROT_WRITE,MAP_SHARED,ps2->spr,0)
        != SPRAM_ADDR)
       DBG("failed to map scratchpad ram",errno)

  /* used for timing events */
  if ((ps2->event = open(PS2_DEV_EVENT,O_RDWR)) < 0)
     DBG("failed to open event",errno)
  else
     ioctl(ps2->event,PS2IOC_ENABLEEVENT,PS2EV_VSYNC);

  return(0);
}

/* clean exit */
int ps2close(ps2hardware ps2h)
{
  close(ps2h.vu0);
  close(ps2h.vu1);
  close(ps2h.pad0);
  close(ps2h.spr);
  close(ps2h.event);
  return(0);
}
